netapp_ontap.resources.software API documentation
Description: Copyright &copy; 2026 NetApp Inc. All rights reserved …

# Module `netapp_ontap.resources.software`

Copyright © 2026 NetApp Inc. All rights reserved.

This file has been automatically generated based on the ONTAP REST API documentation.

## Overview

You can use the ONTAP cluster software API to retrieve and display relevant information about a software profile, software packages collection, software history collection, and firmware packages collection. This API retrieves the information about all software packages present in the cluster, or a specific software package, or firmware upgrade status.
You can use the POST request to download a software package/firmware from an HTTP or FTP server. The PATCH request provides the option to upgrade the cluster software version. Select the `validate_only` field to validate the package before triggering the update. Set the `version` field to trigger the installation of the package in the cluster. You can pause, resume, or cancel any ongoing software upgrade by selecting `action`. You can use the DELETE request to remove a specific software package present in the cluster.

## Examples

### Retrieving software profile information

The following example shows how to retrieve software and firmware profile information. You can check the validation results after selecting the `validate_only` field. Upgrade progress information is available after an upgrade has started.

    from netapp_ontap import HostConnection
    from netapp_ontap.resources import Software

    with HostConnection("<mgmt-ip>", username="admin", password="password", verify=False):
        resource = Software()
        resource.get(return_timeout=15)
        print(resource)

Try it out

    Software(
        {
            "version": "9.5.0",
            "_links": {"self": {"href": "/api/cluster/software/"}},
            "pending_version": "9.6.0",
            "update_details": [
                {
                    "elapsed_duration": 29,
                    "state": "in_progress",
                    "phase": "Data ONTAP updates",
                    "estimated_duration": 4620,
                    "node": {"name": "sti70-vsim-ucs165n"},
                }
            ],
            "state": "in_progress",
            "validation_results": [
                {
                    "issue": {"message": "Use NFS hard mounts, if possible."},
                    "status": "warning",
                    "action": {"message": "Use NFS hard mounts, if possible."},
                    "update_check": "NFS mounts",
                }
            ],
            "metrocluster": {
                "progress_summary": {"message": "Update paused by user"},
                "clusters": [
                    {
                        "estimated_duration": 3480,
                        "state": "waiting",
                        "elapsed_duration": 0,
                        "name": "sti70-vsim-ucs165n_siteA",
                    }
                ],
                "progress_details": {
                    "message": 'Installing software image on cluster "sti70-vsim-ucs165n_siteA".'
                },
            },
            "status_details": [
                {
                    "name": "do-download-job",
                    "start_time": "2018-05-21T09:53:04+05:30",
                    "issue": {"message": "Image update complete", "code": 0},
                    "state": "completed",
                    "end_time": "2018-05-21T11:53:04+05:30",
                    "node": {"name": "sti70-vsim-ucs165n"},
                }
            ],
            "nodes": [
                {
                    "version": "9.5.0",
                    "firmware": {
                        "sp_bmc": {
                            "autoupdate": False,
                            "in_progress": False,
                            "running_version": "1.2.3.4",
                            "image": " primary",
                            "fw_type": "SP",
                            "start_time": "2018-05-21T09:53:04+05:30",
                            "end_time": "2018-05-21T09:53:04+05:30",
                            "is_current": True,
                            "percent_done": 100,
                        },
                        "dqp": {
                            "version": "3.17",
                            "record_count": {
                                "drive": 680,
                                "device": 29,
                                "system": 3,
                                "alias": 200,
                            },
                            "file_name": "qual_devices_v2",
                            "revision": "20200117",
                        },
                        "shelf": {"update_status": "idle", "in_progress_count": 2},
                        "disk": {
                            "update_status": "idle",
                            "num_waiting_download": 0,
                            "average_duration_per_disk": 120,
                            "total_completion_estimate": 0,
                        },
                        "cluster_fw_progress": [
                            {
                                "job": {
                                    "_links": {
                                        "self": {
                                            "href": "/api/cluster/jobs/5a21663c-a9a0-11ea-af9a-005056bb44d7"
                                        }
                                    },
                                    "uuid": "5a21663c-a9a0-11ea-af9a-005056bb44d7",
                                },
                                "zip_file_name": "abc.zip",
                                "update_state": [
                                    {
                                        "message": "Cannot open the local staging zip file.",
                                        "attempts": 3,
                                        "code": 2228325,
                                        "status": "failed",
                                        "worker_node": {
                                            "name": "Node 1",
                                            "uuid": "fcd40f70-f531-11eb-b235-005056bb3497",
                                        },
                                    },
                                    {
                                        "message": "Success",
                                        "attempts": 3,
                                        "code": 0,
                                        "status": "complete",
                                        "worker_node": {
                                            "name": "Node 2",
                                            "uuid": "fcd40f70-f531-11eb-b235-005056bb3498",
                                        },
                                    },
                                ],
                                "update_type": "automatic_update",
                            },
                            {
                                "job": {
                                    "_links": {
                                        "self": {
                                            "href": "/api/cluster/jobs/5a21663c-a9a0-11ea-af9a-005056bb44d7"
                                        }
                                    },
                                    "uuid": "5a21663c-a9a0-11ea-af9a-005056bb44d7",
                                },
                                "zip_file_name": "xyz.zip",
                                "update_state": [
                                    {
                                        "message": "Cannot open the local staging zip file.",
                                        "attempts": 3,
                                        "code": 2228325,
                                        "status": "failed",
                                        "worker_node": {
                                            "name": "Node 1",
                                            "uuid": "fcd40f70-f531-11eb-b235-005056bb3497",
                                        },
                                    },
                                    {
                                        "message": "Success",
                                        "attempts": 3,
                                        "code": 0,
                                        "status": "complete",
                                        "worker_node": {
                                            "name": "Node 2",
                                            "uuid": "fcd40f70-f531-11eb-b235-005056bb3498",
                                        },
                                    },
                                ],
                                "update_type": "manual_update",
                            },
                        ],
                    },
                }
            ],
        }
    )

### Upgrading the software version

The following example shows how to upgrade cluster software. Set the `version` field to trigger the installation of the package. You can select the `validate_only` field to validate the package before the installation starts. Setting `skip_warning` as `true` ignores the validation warning before the installation starts. Setting the `action` field performs a `pause`, `resume`, or `cancel' operation on an ongoing upgrade. An upgrade can only be resumed if it is in the paused state. Setting`stabilize_minutes`allows each node a specified amount of time to stabilize after a reboot; the default is 8 minutes. If`show_validation_details` is set to "true", all validation details will be shown in the output.
You can start the upgrade process at the cluster level. There are no options available to start the upgrade for a specific node or HA pair.

#### 1. Validating the package and verifying the validation results

The following example shows how to validate a cluster software package. You must validate the package before the software upgrade. Set the `validate_only` field to `true` to start the validation. You can check for validation results in the GET /cluster/software endpoint.

    from netapp_ontap import HostConnection
    from netapp_ontap.resources import Software

    with HostConnection("<mgmt_ip>", username="admin", password="password", verify=False):
        resource = Software()
        resource.version = "9.5.0"
        resource.patch(hydrate=True, validate_only=True)

The call to validate the software cluster version returns the job UUID, including a HAL link to retrieve details about the job. The job object includes a `state` field and a message to indicate the progress of the job. When the job is complete and the application is fully created, the message indicates success and the `state` field of the job is set to `success`.

    from netapp_ontap import HostConnection
    from netapp_ontap.resources import Job

    with HostConnection("<mgmt-ip>", username="admin", password="password", verify=False):
        resource = Job(uuid="f587d316-5feb-11e8-b0e0-005056956dfc")
        resource.get()
        print(resource)

Try it out

    Job(
        {
            "message": "success",
            "_links": {
                "self": {"href": "/api/cluster/jobs/f587d316-5feb-11e8-b0e0-005056956dfc"}
            },
            "code": 0,
            "state": "success",
            "uuid": "f587d316-5feb-11e8-b0e0-005056956dfc",
            "description": "PATCH /api/cluster/software",
        }
    )

You can check for validation results in the GET /cluster/software endpoint. The following example shows how to check the validation warnings and errors after setting the `validate_only` field to `true`.

    from netapp_ontap import HostConnection
    from netapp_ontap.resources import Software

    with HostConnection("<mgmt-ip>", username="admin", password="password", verify=False):
        resource = Software()
        resource.get()
        print(resource)

Try it out

    Software(
        {
            "version": "9.7.0",
            "elapsed_duration": 56,
            "_links": {"self": {"href": "/api/cluster/software"}},
            "state": "failed",
            "validation_results": [
                {
                    "issue": {
                        "message": 'Cluster HA is not configured in the cluster. Storage failover is not enabled on node "node1", "node2".'
                    },
                    "status": "error",
                    "action": {
                        "message": "Check cluster HA configuration. Check storage failover status."
                    },
                    "update_check": "High Availability status",
                },
                {
                    "issue": {
                        "message": 'Manual validation checks need to be performed. Refer to the Upgrade Advisor Plan or the "What should I verify before I upgrade with or without Upgrade Advisor" section in the "Upgrade ONTAP" documentation for the remaining validation checks that need to be performed before update. Failing to do so can result in an update failure or an I/O disruption.'
                    },
                    "status": "warning",
                    "action": {
                        "message": 'Refer to the Upgrade Advisor Plan or the "What should I verify before I upgrade with or without Upgrade Advisor" section in the "Upgrade ONTAP" documentation for the remaining validation checks that need to be performed before update.'
                    },
                    "update_check": "Manual checks",
                },
            ],
            "estimated_duration": 600,
            "nodes": [
                {
                    "version": "9.5.0",
                    "firmware": {
                        "sp_bmc": {
                            "autoupdate": False,
                            "in_progress": False,
                            "running_version": "1.2.3.4",
                            "image": " primary",
                            "fw_type": "SP",
                            "start_time": "2018-05-21T09:53:04+05:30",
                            "end_time": "2018-05-21T09:53:04+05:30",
                            "is_current": True,
                            "percent_done": 100,
                        },
                        "dqp": {
                            "version": "3.17",
                            "record_count": {
                                "drive": 680,
                                "device": 29,
                                "system": 3,
                                "alias": 200,
                            },
                            "file_name": "qual_devices_v2",
                            "revision": "20200117",
                        },
                        "shelf": {"update_status": "idle", "in_progress_count": 2},
                        "disk": {
                            "update_status": "idle",
                            "num_waiting_download": 0,
                            "average_duration_per_disk": 120,
                            "total_completion_estimate": 0,
                        },
                        "cluster_fw_progress": [
                            {
                                "job": {
                                    "_links": {
                                        "self": {
                                            "href": "/api/cluster/jobs/5a21663c-a9a0-11ea-af9a-005056bb44d7"
                                        }
                                    },
                                    "uuid": "5a21663c-a9a0-11ea-af9a-005056bb44d7",
                                },
                                "zip_file_name": "abc.zip",
                                "update_state": [
                                    {
                                        "message": "Cannot open the local staging zip file.",
                                        "attempts": 3,
                                        "code": 2228325,
                                        "status": "failed",
                                        "worker_node": {
                                            "name": "Node 1",
                                            "uuid": "fcd40f70-f531-11eb-b235-005056bb3498",
                                        },
                                    },
                                    {
                                        "message": "Success",
                                        "attempts": 3,
                                        "code": 0,
                                        "status": "complete",
                                        "worker_node": {
                                            "name": "Node 2",
                                            "uuid": "fcd40f70-f531-11eb-b235-005056bb3498",
                                        },
                                    },
                                ],
                                "update_type": "automatic_update",
                            },
                            {
                                "job": {
                                    "_links": {
                                        "self": {
                                            "href": "/api/cluster/jobs/5a21663c-a9a0-11ea-af9a-005056bb44d7"
                                        }
                                    },
                                    "uuid": "5a21663c-a9a0-11ea-af9a-005056bb44d7",
                                },
                                "zip_file_name": "xyz.zip",
                                "update_state": [
                                    {
                                        "message": "Cannot open the local staging zip file.",
                                        "attempts": 3,
                                        "code": 2228325,
                                        "status": "failed",
                                        "worker_node": {
                                            "name": "Node 1",
                                            "uuid": "fcd40f70-f531-11eb-b235-005056bb3498",
                                        },
                                    },
                                    {
                                        "message": "Success",
                                        "attempts": 3,
                                        "code": 0,
                                        "status": "complete",
                                        "worker_node": {
                                            "name": "Node 2",
                                            "uuid": "fcd40f70-f531-11eb-b235-005056bb3498",
                                        },
                                    },
                                ],
                                "update_type": "automatic_update",
                            },
                        ],
                    },
                }
            ],
        }
    )

#### 2. Updating the cluster

The following example shows how to initiate a cluster software upgrade. You must validate the package before the software upgrade starts. Set the `skip_warnings` field to `true` to skip validation warnings and start the software package upgrade. You can specify the `stabilize_minutes` value between 1 to 60 minutes. Setting `stabilize_minutes` allows each node a specified amount of time to stabilize after a reboot; the default is 8 minutes. If the value of `show_validation_details` is set to "true", then all validation details will be shown in the output. By default, on non-MetroCluster configurations, nodes at the target release will be skipped over. However, this can be disabled by setting `skip_nodes_at_target_version` to `false`. Note: it is invalid to set `skip_nodes_at_target_version` to `true` in pause, resume, or cancel actions. `skip_nodes_at_target_version` cannot be set to `true` in MetroCluster configurations.

    from netapp_ontap import HostConnection
    from netapp_ontap.resources import Software

    with HostConnection("<mgmt_ip>", username="admin", password="password", verify=False):
        resource = Software()
        resource.version = "9.5.0"
        resource.patch(hydrate=True, skip_warnings=True)

The call to update the software cluster version returns the job UUID, including a HAL link to retrieve details about the job. The job object includes a `state` field and a message to indicate the progress of the job. When the job is complete and the application is fully created, the message indicates success and the `state` field of the job is set to `success`.

    from netapp_ontap import HostConnection
    from netapp_ontap.resources import Job

    with HostConnection("<mgmt-ip>", username="admin", password="password", verify=False):
        resource = Job(uuid="f587d316-5feb-11e8-b0e0-005056956dfc")
        resource.get()
        print(resource)

Try it out

    Job(
        {
            "message": "success",
            "_links": {
                "self": {"href": "/api/cluster/jobs/f587d316-5feb-11e8-b0e0-005056956dfc"}
            },
            "code": 0,
            "state": "success",
            "uuid": "f587d316-5feb-11e8-b0e0-005056956dfc",
            "description": "PATCH /api/cluster/software",
        }
    )

You can check the update progress information in the GET /cluster/software endpoint. The following example shows how to check the progress of an update after setting the `skip_warnings` field to `true`. Each node's object also includes information about the firmware update status on the node.

    from netapp_ontap import HostConnection
    from netapp_ontap.resources import Software

    with HostConnection("<mgmt-ip>", username="admin", password="password", verify=False):
        resource = Software()
        resource.get()
        print(resource)

Try it out

    Software(
        {
            "version": "9.7.0",
            "elapsed_duration": 63,
            "_links": {"self": {"href": "/api/cluster/software"}},
            "pending_version": "9.7.0",
            "update_details": [
                {
                    "elapsed_duration": 10,
                    "phase": "Data ONTAP updates",
                    "estimated_duration": 4620,
                    "node": {"name": "node1"},
                },
                {
                    "elapsed_duration": 10,
                    "phase": "Data ONTAP updates",
                    "estimated_duration": 4620,
                    "node": {"name": "node2"},
                },
            ],
            "state": "in_progress",
            "validation_results": [
                {
                    "issue": {
                        "message": 'Manual validation checks need to be performed. Refer to the Upgrade Advisor Plan or the "What should I verify before I upgrade with or without Upgrade Advisor" section in the "Upgrade ONTAP" documentation for the remaining validation checks that need to be performed before update. Failing to do so can result in an update failure or an I/O disruption.'
                    },
                    "status": "warning",
                    "action": {
                        "message": 'Refer to the Upgrade Advisor Plan or the "What should I verify before I upgrade with or without Upgrade Advisor" section in the "Upgrade ONTAP" documentation for the remaining validation checks that need to be performed before update.'
                    },
                    "update_check": "Manual checks",
                }
            ],
            "status_details": [
                {
                    "name": "do-download-job",
                    "start_time": "2019-01-14T23:12:14+05:30",
                    "issue": {"message": "Installing software image.", "code": 10551400},
                    "end_time": "2019-01-14T23:12:14+05:30",
                    "node": {"name": "node1"},
                },
                {
                    "name": "do-download-job",
                    "start_time": "2019-01-14T23:12:14+05:30",
                    "issue": {"message": "Installing software image.", "code": 10551400},
                    "end_time": "2019-01-14T23:12:14+05:30",
                    "node": {"name": "node2"},
                },
            ],
            "estimated_duration": 5220,
            "nodes": [
                {
                    "version": "9.5.0",
                    "firmware": {
                        "sp_bmc": {
                            "autoupdate": False,
                            "in_progress": False,
                            "running_version": "1.2.3.4",
                            "image": " primary",
                            "fw_type": "SP",
                            "start_time": "2018-05-21T09:53:04+05:30",
                            "end_time": "2018-05-21T09:53:04+05:30",
                            "is_current": True,
                            "percent_done": 100,
                        },
                        "dqp": {
                            "version": "3.17",
                            "record_count": {
                                "drive": 680,
                                "device": 29,
                                "system": 3,
                                "alias": 200,
                            },
                            "file_name": "qual_devices_v2",
                            "revision": "20200117",
                        },
                        "shelf": {"update_status": "idle", "in_progress_count": 2},
                        "disk": {
                            "update_status": "idle",
                            "num_waiting_download": 0,
                            "average_duration_per_disk": 120,
                            "total_completion_estimate": 0,
                        },
                        "cluster_fw_progress": [
                            {
                                "job": {
                                    "_links": {
                                        "self": {
                                            "href": "/api/cluster/jobs/5a21663c-a9a0-11ea-af9a-005056bb44d7"
                                        }
                                    },
                                    "uuid": "5a21663c-a9a0-11ea-af9a-005056bb44d7",
                                },
                                "zip_file_name": "abc.zip",
                                "update_state": [
                                    {
                                        "message": "Cannot open the local staging zip file.",
                                        "attempts": 3,
                                        "code": 2228325,
                                        "status": "failed",
                                        "worker_node": {
                                            "name": "Node 3",
                                            "uuid": "fcd40f70-f531-11eb-b235-005056bb3497",
                                        },
                                    },
                                    {
                                        "message": "Success",
                                        "attempts": 3,
                                        "code": 0,
                                        "status": "complete",
                                        "worker_node": {
                                            "name": "Node 4",
                                            "uuid": "fcd40f70-f531-11eb-b235-005056bb3498",
                                        },
                                    },
                                ],
                                "update_type": "automated_update",
                            },
                            {
                                "job": {
                                    "_links": {
                                        "self": {
                                            "href": "/api/cluster/jobs/5a21663c-a9a0-11ea-af9a-005056bb44d7"
                                        }
                                    },
                                    "uuid": "5a21663c-a9a0-11ea-af9a-005056bb44d7",
                                },
                                "zip_file_name": "xyz.zip",
                                "update_state": [
                                    {
                                        "message": "Cannot open the local staging zip file.",
                                        "attempts": 3,
                                        "code": 2228325,
                                        "status": "failed",
                                        "worker_node": {
                                            "name": "Node 1",
                                            "uuid": "fcd40f70-f531-11eb-b235-005056bb3497",
                                        },
                                    },
                                    {
                                        "message": "Success",
                                        "attempts": 3,
                                        "code": 0,
                                        "status": "complete",
                                        "worker_node": {
                                            "name": "Node 2",
                                            "uuid": "fcd40f70-f531-11eb-b235-005056bb3497",
                                        },
                                    },
                                ],
                                "update_type": "automated_update",
                            },
                        ],
                    },
                }
            ],
        }
    )

In the case of a post update check failure, the details are available under the heading "post_update_checks" in the GET /cluster/software endpoint. The following example shows how to check the progress of an update after a post update check has failed. Each node's object also includes information about the firmware update status on the node.

    from netapp_ontap import HostConnection
    from netapp_ontap.resources import Software

    with HostConnection("<mgmt-ip>", username="admin", password="password", verify=False):
        resource = Software()
        resource.get()
        print(resource)

Try it out

    Software(
        {
            "version": "9.7.0",
            "elapsed_duration": 63,
            "_links": {"self": {"href": "/api/cluster/software"}},
            "pending_version": "9.7.0",
            "update_details": [
                {
                    "elapsed_duration": 3120,
                    "phase": "Data ONTAP updates",
                    "estimated_duration": 4620,
                    "node": {"name": "node1"},
                },
                {
                    "elapsed_duration": 3210,
                    "phase": "Data ONTAP updates",
                    "estimated_duration": 4620,
                    "node": {"name": "node2"},
                },
                {
                    "elapsed_duration": 10,
                    "phase": "Post-update checks",
                    "estimated_duration": 600,
                    "node": {"name": "node2"},
                },
            ],
            "state": "in_progress",
            "validation_results": [
                {
                    "issue": {
                        "message": 'Manual validation checks need to be performed. Refer to the Upgrade Advisor Plan or the "What should I verify before I upgrade with or without Upgrade Advisor" section in the "Upgrade ONTAP" documentation for the remaining validation checks that need to be performed before update. Failing to do so can result in an update failure or an I/O disruption.'
                    },
                    "status": "warning",
                    "action": {
                        "message": 'Refer to the Upgrade Advisor Plan or the "What should I verify before I upgrade with or without Upgrade Advisor" section in the "Upgrade ONTAP" documentation for the remaining validation checks that need to be performed before update.'
                    },
                    "update_check": "Manual checks",
                }
            ],
            "post_update_checks": [
                {
                    "issue": {"message": "Not all aggregates are online"},
                    "status": "error",
                    "action": {"message": "Ensure all aggregates are online."},
                    "update_check": "Aggregate Health Status",
                },
                {
                    "issue": {
                        "message": "Storage failover is not enabled on nodes of the cluster."
                    },
                    "status": "error",
                    "action": {
                        "message": "Ensure storage failover is enabled on all nodes of the cluster."
                    },
                    "update_check": "HA Health Status",
                },
            ],
            "status_details": [
                {
                    "name": "do-download-job",
                    "start_time": "2019-01-14T23:12:14+05:30",
                    "issue": {"message": "Image update complete.", "code": 0},
                    "end_time": "2019-01-14T23:12:14+05:30",
                    "node": {"name": "node1"},
                },
                {
                    "name": "do-download-job",
                    "start_time": "2019-01-14T23:12:14+05:30",
                    "issue": {"message": "Image update complete.", "code": 0},
                    "end_time": "2019-01-14T23:12:14+05:30",
                    "node": {"name": "node2"},
                },
            ],
            "estimated_duration": 5220,
            "nodes": [
                {
                    "version": "9.5.0",
                    "firmware": {
                        "sp_bmc": {
                            "autoupdate": False,
                            "in_progress": True,
                            "running_version": "1.2.3.4",
                            "image": " primary",
                            "fw_type": "SP",
                            "start_time": "2018-05-21T09:53:04+05:30",
                            "end_time": "2018-05-21T09:53:04+05:30",
                            "is_current": True,
                            "percent_done": 100,
                        },
                        "dqp": {
                            "version": "3.17",
                            "record_count": {
                                "drive": 680,
                                "device": 29,
                                "system": 3,
                                "alias": 200,
                            },
                            "file_name": "qual_devices_v2",
                            "revision": "20200117",
                        },
                        "shelf": {"update_status": "idle", "in_progress_count": 2},
                        "disk": {
                            "update_status": "idle",
                            "num_waiting_download": 0,
                            "average_duration_per_disk": 120,
                            "total_completion_estimate": 0,
                        },
                        "cluster_fw_progress": [
                            {
                                "job": {
                                    "_links": {
                                        "self": {
                                            "href": "/api/cluster/jobs/5a21663c-a9a0-11ea-af9a-005056bb44d7"
                                        }
                                    },
                                    "uuid": "5a21663c-a9a0-11ea-af9a-005056bb44d7",
                                },
                                "zip_file_name": "abc.zip",
                                "update_state": [
                                    {
                                        "message": "<message catalog text>",
                                        "attempts": 3,
                                        "code": 3,
                                        "status": "working",
                                        "worker_node": {
                                            "name": "Node 1",
                                            "uuid": "fcd40f70-f531-11eb-b235-005056bb3497",
                                        },
                                    },
                                    {
                                        "message": "Error message",
                                        "attempts": 3,
                                        "code": 0,
                                        "status": "completed",
                                        "worker_node": {
                                            "name": "Node 2",
                                            "uuid": "fcd40f70-f531-11eb-b235-005056bb3497",
                                        },
                                    },
                                ],
                                "update_type": "automated_update",
                            },
                            {
                                "job": {
                                    "_links": {
                                        "self": {
                                            "href": "/api/cluster/jobs/5a21663c-a9a0-11ea-af9a-005056bb44d7"
                                        }
                                    },
                                    "uuid": "5a21663c-a9a0-11ea-af9a-005056bb44d7",
                                },
                                "zip_file_name": "xyz.zip",
                                "update_state": [
                                    {
                                        "message": "Error message",
                                        "attempts": 1,
                                        "code": 0,
                                        "status": "completed",
                                        "worker_node": {
                                            "name": "Node 1",
                                            "uuid": "fcd40f70-f531-11eb-b235-005056bb3497",
                                        },
                                    },
                                    {
                                        "message": "Error message",
                                        "attempts": 3,
                                        "code": 0,
                                        "status": "completed",
                                        "worker_node": {
                                            "name": "Node 2",
                                            "uuid": "fcd40f70-f531-11eb-b235-005056bb3497",
                                        },
                                    },
                                ],
                                "update_type": "automated_update",
                            },
                        ],
                    },
                }
            ],
        }
    )

#### 3. Pausing, resuming or canceling an upgrade

The following example shows how to `pause` an ongoing cluster software package upgrade. Set the `action` field to `pause`, `resume`, or `cancel` to pause, resume or cancel the upgrade respectively. Not all update operations support these actions. An update can only be resumed if it is in the paused state.

    from netapp_ontap import HostConnection
    from netapp_ontap.resources import Software

    with HostConnection("<mgmt_ip>", username="admin", password="password", verify=False):
        resource = Software()
        resource.version = "9.5.0"
        resource.patch(hydrate=True, action="pause")

The call to update the software cluster version and/or firmware version returns the job UUID, including a HAL link to retrieve details about the job. The job object includes a `state` field and a message to indicate the progress of the job. When the job is complete and the application is fully created, the message indicates success and the `state` field of the job is set to `success`.

    from netapp_ontap import HostConnection
    from netapp_ontap.resources import Job

    with HostConnection("<mgmt-ip>", username="admin", password="password", verify=False):
        resource = Job(uuid="f587d316-5feb-11e8-b0e0-005056956dfc")
        resource.get()
        print(resource)

Try it out

    Job(
        {
            "message": "success",
            "_links": {
                "self": {"href": "/api/cluster/jobs/f587d316-5feb-11e8-b0e0-005056956dfc"}
            },
            "code": 0,
            "state": "success",
            "uuid": "f587d316-5feb-11e8-b0e0-005056956dfc",
            "description": "PATCH /api/cluster/software",
        }
    )

You can check the progress of the upgrade in the GET /cluster/software endpoint. The following example shows how to check the progress of the pause upgrade state after setting the `action` field to `pause`.

    from netapp_ontap import HostConnection
    from netapp_ontap.resources import Software

    with HostConnection("<mgmt-ip>", username="admin", password="password", verify=False):
        resource = Software()
        resource.get()
        print(resource)

Try it out

    Software(
        {
            "version": "9.7.0",
            "elapsed_duration": 103,
            "_links": {"self": {"href": "/api/cluster/software"}},
            "pending_version": "9.7.0",
            "update_details": [
                {
                    "elapsed_duration": 54,
                    "phase": "Pre-update checks",
                    "estimated_duration": 600,
                    "node": {"name": "node1"},
                },
                {
                    "elapsed_duration": 49,
                    "phase": "Data ONTAP updates",
                    "estimated_duration": 4620,
                    "node": {"name": "node2"},
                },
                {
                    "elapsed_duration": 49,
                    "phase": "Data ONTAP updates",
                    "estimated_duration": 4620,
                },
            ],
            "state": "pause_pending",
            "validation_results": [
                {
                    "issue": {
                        "message": 'Manual validation checks need to be performed. Refer to the Upgrade Advisor Plan or the "What should I verify before I upgrade with or without Upgrade Advisor" section in the "Upgrade ONTAP" documentation for the remaining validation checks that need to be performed before update. Failing to do so can result in an update failure or an I/O disruption.'
                    },
                    "status": "warning",
                    "action": {
                        "message": 'Refer to the Upgrade Advisor Plan or the "What should I verify before I upgrade with or without Upgrade Advisor" section in the "Upgrade ONTAP" documentation for the remaining validation checks that need to be performed before update.'
                    },
                    "update_check": "Manual checks",
                }
            ],
            "status_details": [
                {
                    "start_time": "2019-01-08T02:54:36+05:30",
                    "issue": {"message": "Installing software image.", "code": 10551400},
                    "node": {"name": "node1"},
                },
                {
                    "start_time": "2019-01-08T02:54:36+05:30",
                    "issue": {"message": "Installing software image.", "code": 10551400},
                    "node": {"name": "node2"},
                },
            ],
            "estimated_duration": 5220,
            "nodes": [
                {
                    "version": "9.5.0",
                    "firmware": {
                        "sp_bmc": {
                            "autoupdate": False,
                            "in_progress": False,
                            "running_version": "1.2.3.4",
                            "image": " primary",
                            "fw_type": "SP",
                            "start_time": "2018-05-21T09:53:04+05:30",
                            "end_time": "2018-05-21T09:53:04+05:30",
                            "is_current": True,
                            "percent_done": 100,
                        },
                        "dqp": {
                            "version": "3.17",
                            "record_count": {
                                "drive": 680,
                                "device": 29,
                                "system": 3,
                                "alias": 200,
                            },
                            "file_name": "qual_devices_v2",
                            "revision": "20200117",
                        },
                        "shelf": {"update_status": "idle", "in_progress_count": 2},
                        "disk": {
                            "update_status": "idle",
                            "num_waiting_download": 0,
                            "average_duration_per_disk": 120,
                            "total_completion_estimate": 0,
                        },
                        "cluster_fw_progress": [
                            {
                                "job": {
                                    "_links": {
                                        "self": {
                                            "href": "/api/cluster/jobs/5a21663c-a9a0-11ea-af9a-005056bb44d7"
                                        }
                                    },
                                    "uuid": "5a21663c-a9a0-11ea-af9a-005056bb44d7",
                                },
                                "zip_file_name": "abc.zip",
                                "update_state": [
                                    {
                                        "message": "Cannot open the local staging zip file.",
                                        "attempts": 3,
                                        "code": 2228325,
                                        "status": "failed",
                                        "worker_node": {
                                            "name": "Node 1",
                                            "uuid": "fcd40f70-f531-11eb-b235-005056bb3497",
                                        },
                                    },
                                    {
                                        "message": "Success",
                                        "attempts": 3,
                                        "code": 0,
                                        "status": "complete",
                                    },
                                ],
                                "update_type": "automated_update",
                            },
                            {
                                "job": {
                                    "_links": {
                                        "self": {
                                            "href": "/api/cluster/jobs/5a21663c-a9a0-11ea-af9a-005056bb44d7"
                                        }
                                    },
                                    "uuid": "5a21663c-a9a0-11ea-af9a-005056bb44d7",
                                },
                                "zip_file_name": "xyz.zip",
                                "update_state": [
                                    {
                                        "message": "Cannot open the local staging zip file.",
                                        "attempts": 3,
                                        "code": 2228325,
                                        "status": "failed",
                                        "worker_node": {
                                            "name": "Node 1",
                                            "uuid": "fcd40f70-f531-11eb-b235-005056bb3497",
                                        },
                                    },
                                    {
                                        "message": "Success",
                                        "attempts": 3,
                                        "code": 0,
                                        "status": "complete",
                                    },
                                ],
                                "update_type": "automated_update",
                            },
                        ],
                    },
                }
            ],
        }
    )

### Uploading a software/firmware package

The following example shows how to upload a software package.

    from netapp_ontap import HostConnection
    from netapp_ontap.resources import Software

    with HostConnection(
        "<mgmt-ip>", username="username", password="password", verify=False
    ):
        resource = Software()
        resource.upload()

Try it out

    Software({})

## Classes

`class Software (*args, **kwargs)`

  Allows interaction with Software objects on the host

  Initialize the instance of the resource.

  Any keyword arguments are set on the instance as properties. For example, if the class was named 'MyResource', then this statement would be true:

      MyResource(name='foo').name == 'foo'

  ## Args

  `*args`
    Each positional argument represents a parent key as used in the URL of the object. That is, each value will be used to fill in a segment of the URL which refers to some parent object. The order of these arguments must match the order they are specified in the URL, from left to right.
  `**kwargs`
    each entry will have its key set as an attribute name on the instance and its value will be the value of that attribute.

  ### Ancestors

  - Resource

  ### Methods

  `def get (self, **kwargs) -> NetAppResponse`

    Retrieves the software profile of a cluster.

    ### Related ONTAP commands

    - `cluster image show`
    - `cluster image show-update-progress`
    - `system node image package show`

    ### Learn more

    - `DOC /cluster/software`

    Fetch the details of the object from the host.

    Requires the keys to be set (if any). After returning, new or changed properties from the host will be set on the instance.

    ## Returns

    A `NetAppResponse` object containing the details of the HTTP response.

    ## Raises

    `NetAppRestError`: If the API call returned a status code >= 400 or if not all of the keys required are present and config.STRICT_GET has been set to True.

  `def patch (self, hydrate: bool = False, poll: bool = True, poll_interval: Optional[int] = None, poll_timeout: Optional[int] = None, **kwargs) -> NetAppResponse`

    Updates the cluster software version. Important note: * Setting 'version' triggers the package installation. * To validate the package for installation but not perform the installation, use the `validate_only` field on the request.

    ### Required properties

    - `version` - Software version to be installed on the cluster.

    ### Recommended optional parameters

    - `validate_only` - Required to validate a software package before an upgrade.
    - `skip_warnings` - Used to skip validation warnings when starting a software upgrade.
    - `action` - Used to pause, resume, or cancel an ongoing software upgrade.
    - `stabilize_minutes` - Specifies a custom value between 1 to 60 minutes that allows each node a specified amount of time to stabilize after a reboot; the default is 8 minutes.
    - `estimate_only` - Estimates the time duration; does not perform any update.
    - `nodes_to_update` - Specifies a subset of the cluster's nodes for update.
    - `show_validation_details` - If the value is set to true, then all validation details will be shown in the output.
    - `skip_nodes_at_target_version` - If the value is set to true, then nodes already at the target version will not be upgraded. Defaults to true in non-MetroCluster configurations. Setting this option to false will force all the selected nodes of the cluster to undergo upgrade.

    ### Related ONTAP commands

    - `cluster image validate`
    - `cluster image update`
    - `cluster image pause-update`
    - `cluster image resume-update`
    - `cluster image cancel-update`

    ### Learn more

    - `DOC /cluster/software`

    Send the difference in the object's state to the host as a modification request.

    Calculates the difference in the object's state since the last time we interacted with the host and sends this in the request body.

    ## Args

    `hydrate`
      If set to True, after the response is received from the call, a a GET call will be made to refresh all fields of the object.
    `poll`
      If set to True, the call will not return until the asynchronous job on the host has completed. Has no effect if the host did not return a job response.
    `poll_interval`
      If the operation returns a job, this specifies how often to query the job for updates.
    `poll_timeout`
      If the operation returns a job, this specifies how long to continue monitoring the job's status for completion.
    `connection`
      The `HostConnection` object to use for this API call. If unset, tries to use the connection which is set globally for the library or from the current context.
    `**kwargs`
      Any key/value pairs passed will normally be sent as query parameters to the host. If any of these pairs are parameters that are sent as formdata then only parameters of that type will be accepted and all others will be discarded.

    ## Returns

    A `NetAppResponse` object containing the details of the HTTP response.

    ## Raises

    `NetAppRestError`: If the API call returned a status code >= 400

  `def upload (self, body: Union[Resource, dict] = None, poll: bool = True, poll_interval: Optional[int] = None, poll_timeout: Optional[int] = None, **kwargs) -> NetAppResponse`

    Uploads a software or firmware package located on the local filesystem.

    ### Related ONTAP commands

    - `cluster image package get`

    ### Learn more

    - `DOC /cluster/software`

    Perform a custom action on this resource which is not a simple CRUD action

    ## Args

    `path`
      The action verb for this request. This will be added as a postfix to the instance location of the resource.
    `body`
      The body of the action request. This should be a Resource instance. The connection and URL will be determined based on the values from this object.
    `poll`
      If set to True, the call will not return until the asynchronous job on the host has completed. Has no effect if the host did not return a job response.
    `poll_interval`
      If the operation returns a job, this specifies how often to query the job for updates.
    `poll_timeout`
      If the operation returns a job, this specifies how long to continue monitoring the job's status for completion.
    `**kwargs`
      Any key/value pairs passed will be sent as query parameters to the host.

    ## Returns

    A `NetAppResponse` object containing the details of the HTTP response.

    ## Raises

    `NetAppRestError`: If the API call returned a status code >= 400

  ### Inherited members

  - `Resource`:
    - `from_dict`
    - `get_collection_url`
    - `get_connection`
    - `instance_location`
    - `path_keys`
    - `poll`
    - `set_connection`
    - `to_dict`
    - `validate_schema`
`class SoftwareSchema (*, only: types.StrSequenceOrSet | None = None, exclude: types.StrSequenceOrSet = (), many: bool | None = None, context: dict | None = None, load_only: types.StrSequenceOrSet = (), dump_only: types.StrSequenceOrSet = (), partial: bool | types.StrSequenceOrSet | None = None, unknown: str | None = None)`

  The fields of the Software object

  ### Ancestors

  - netapp_ontap.resource.ResourceSchema
  - marshmallow.schema.Schema
  - marshmallow.base.SchemaABC
  - abc.ABC

  ### Class variables

  `action: str GET POST PATCH`

    User triggered action to apply to the install operation

    Valid choices:

    - pause
    - cancel
    - resume
  `elapsed_duration: Size GET`

    Elapsed time during the upgrade or validation operation

    Example: 2140

  `estimated_duration: Size GET`

    Overall estimated time for completion of the upgrade or validation operation.

    Example: 5220

  `links: SelfLink GET`

    The links field of the software.

  `metrocluster: SoftwareReferenceMetrocluster GET POST PATCH`

    The metrocluster field of the software.

  `nodes: List[SoftwareNode] GET`

    List of nodes, active versions, and firmware update progressions.

  `pending_version: str GET`

    Version being installed on the system.

    Example: ONTAP_X_1

  `post_update_checks: List[SoftwareValidation] GET`

    List of failed post-update checks' warnings, errors, and advice.

  `state: str GET`

    Operational state of the upgrade

    Valid choices:

    - in_progress
    - waiting
    - paused_by_user
    - paused_on_error
    - completed
    - canceled
    - failed
    - pause_pending
    - cancel_pending
  `status_details: List[SoftwareStatusDetails] GET`

    Display status details.

  `update_details: List[SoftwareUpdateDetails] GET`

    Display update progress details.

  `validation_results: List[SoftwareValidation] GET`

    List of validation warnings, errors, and advice.

  `version: str GET POST PATCH`

    Version of ONTAP installed and currently active on the system. During PATCH, using the 'validate_only' parameter on the request executes pre-checks, but does not perform the full installation.

    Example: ONTAP_X